<html>
<head>
  <link rel="stylesheet" type="text/css" href="template.css" />
  <!-- for font -->
<link href="https://fonts.googleapis.com/css?family=Economica:700" rel="stylesheet">
  <title>{{article.title}}</title>
</head>
<body>


</body>
</html>
<div id='logo'>
  <h1 id='logo'>
    <a href='index.html'>{{site_config.title}}</a>
  </h1>
</div>

<div id='content'>
  {{article.content}}
</div>

<div id='footer'>
  {{site_config.copyright}}
</div>
