body {
  font-family: Arial;
  font-size: 18px;
  line-height: 1.8em;
}
h1#logo a {
  height: 2em;
  color: #ffffff;
}



a {
  text-decoration: none;
}
a:link {
  color: black;
}
a:visited {
  color:  black;
}

div#logo {
  font-family: 'Economica', sans-serif;
  text-align: center;
  line-height: 3em;
  width: 100%;
  height: 5 em;
  background-color: {{favorite_color}};
}

pre {
  padding: 0;
  margin: 0;
}



div#content {
  margin-left: 15%;
  margin-right: 15%;
  margin-bottom: 1em;
  padding: 0.5em;
}

div#footer {
  background-color: black;
  line-height: 2em;
  height: 2em;
  padding: 1em;
  color: white;
  text-align: center;
  font-size: 0.7em;
}

.code_block {
  font-size: 0.8em;
  border: 0;
}

.common_box {
  background-color: #f1f1f1;
  border-radius: 5px;
  padding: 0.5em;


}

div.article_box {
  border: solid 1px;
}

h2.article_box_article_title {
  font-family: 'Economica', sans-serif;
  padding: 0px;
  margin: 0px;
}

span.article_box_tag {

}

p.article_box_body {

}

div.article_box_button_read {
  background-color: #9999CC;
  padding: 0.1em;
  border-radius: 5px;
  width: 6em;
  text-align: center;
}
div.article_box_button_read a {
  color: white;
}

div.definition_box {
    position: relative;
    margin: 2em 0;
    padding: 0.5em 1em;
    border: solid 1px;

}
span.definition_box_title {
    position: absolute;
    display: inline-block;
    top: -13px;
    left: 10px;
    padding: 0 9px;
    line-height: 1;
    font-size: 19px;
    background: #FFF;
    font-weight: bold;
}
p.definition_box_body {
    margin: 0;
    padding: 0;
}

span.definition_box_flag {

}

blockquote {
  border-left: 3px solid gray;
  background-color:  #f1f1f1;
  margin: 0;
  padding-left   : 10px; 
  padding-top    :  5px; 
  padding-bottom :  5px; 

}